searchData={"items":[{"type":"module","title":"Membrane.RTC.Engine.Endpoint.WebRTC","doc":"An Endpoint responsible for communicatiing with WebRTC client.\n\nIt is responsible for sending and receiving media tracks from other WebRTC clients (e.g. web browsers).","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.html"},{"type":"module","title":"Signalling - Membrane.RTC.Engine.Endpoint.WebRTC","doc":"In order to operate correctly, this endpoint requires that the business logic implements\nthe signalling channel to the client it represents. Implementation details are not important to\nendpoint.\n\nSignalling in WebRTC is used to eg. facilitate SDP negotiation or send notifications regarding the session.\nThis endpoint uses an abstraction of Media Events to achieve this.\nA Media Event is a black box message sent between the Client Library and this endpoint.\n\nThe business logic will receive an encoded media event in binary form from both the endpoint and the client library.\nIt's only responsibility is to feed the binary to either the endpoint or the client library.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.html#module-signalling"},{"type":"module","title":"Message Protocol Definition - Membrane.RTC.Engine.Endpoint.WebRTC","doc":"The business logic must receive `t:media_event_message_t/0` message from the endpoint and forward it to the client\nover the implemented signalling channel. Mind you, this message will be wrapped with `t:Membrane.RTC.Engine.Message.EndpointMessage.t/0`\n\n**Example**\n```elixir\n@impl GenServer\ndef handle_info(%Membrane.RTC.Engine.Message.EndpointMessage{endpoint_id: endpoint, message: {:media_event, _event} = message}, state) do\n  send state.channels[endpoint], message\n  {:noreply, state}\nend\n```\n\nSimmilarily, it must forward all media events it receives over the signalling channel to the endpoint\nby sending a `t:media_event_message_t/0` to it.\n\nExample.\n```elixir\n@impl GenServer\ndef handle_info({:media_event, origin, event}, state) do\n  endpoint_id = state.client_channel_to_endpoint_id[origin]\n  Engine.message_endpoint(endpoint_id, {:media_event, event})\n  {:noreply, state}\nend\n```","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.html#module-message-protocol-definition"},{"type":"module","title":"Client libraries - Membrane.RTC.Engine.Endpoint.WebRTC","doc":"The following client libraries for handling signalling messages are available:\n* [Typescript library intended for use in web browsers](https://github.com/membraneframework/membrane-webrtc-js)\n* [Android native library](https://github.com/membraneframework/membrane-webrtc-android)\n* [IOS native library](https://github.com/membraneframework/membrane-webrtc-ios)","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.html#module-client-libraries"},{"type":"module","title":"Monitoring track activity - Membrane.RTC.Engine.Endpoint.WebRTC","doc":"WebRTC Endpoint only monitors simulcast tracks activity, meaning that it never emits `Membrane.RTC.Engine.Event.TrackVariantPaused` event\nfor non-simulcast tracks. The main reason is that it's impossible to tell if the screensharing track is inactive or not sending any packets because it contains static content.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.html#module-monitoring-track-activity"},{"type":"module","title":"Bin options - Membrane.RTC.Engine.Endpoint.WebRTC","doc":"Passed via struct `t:Membrane.RTC.Engine.Endpoint.WebRTC.t/0`\n\n- `rtc_engine`  \n\n  ```\n  pid()\n  ```\n  \n  ***Required***  \n  Pid of parent Engine\n\n\n- `direction`  \n\n  ```\n  EndpointBin.direction()\n  ```\n  \n  Default value: `:sendrecv`  \n  Direction of WebRTC Endpoint. Determines whether\n  EndpointBin can send, receive or both send and receive media.\n  For more information refer to t:EndpointBin.direction/0.\n\n\n\n- `ice_name`  \n\n  ```\n  String.t()\n  ```\n  \n  ***Required***  \n  Ice name is used in creating credentials for ice connnection\n\n\n\n- `handshake_opts`  \n\n  ```\n  Keyword.t()\n  ```\n  \n  Default value: `[]`  \n  Keyword list with options for handshake module. For more information please\n  refer to `Membrane.ICE.Bin`\n\n\n\n- `filter_codecs`  \n\n  ```\n  ({RTPMapping.t(), FMTP.t() | nil} -> boolean())\n  ```\n  \n  Default value: `&Membrane.WebRTC.SDP.filter_encodings/1`  \n  Defines function which will filter SDP m-line by codecs\n\n\n\n- `log_metadata`  \n\n  ```\n  :list\n  ```\n  \n  Default value: `[]`  \n  Logger metadata used for endpoint bin and all its descendants\n\n\n\n- `webrtc_extensions`  \n\n  ```\n  [module()]\n  ```\n  \n  Default value: `[]`  \n  List of WebRTC extensions to use.\n  \n  Each module has to implement `Membrane.WebRTC.Extension.t()`.\n\n\n\n- `extensions`  \n\n  ```\n  %{(encoding_name :: atom() | :any) => [Membrane.RTP.SessionBin.extension_t()]}\n  ```\n  \n  Default value: `%{}`  \n  A map pointing from encoding names to lists of extensions that should be used for given encodings.\n  Encoding \"`:any`\" indicates that extensions should be applied regardless of encoding.\n  \n  A sample usage would be to add silence discarder to OPUS tracks when VAD extension is enabled.\n  It can greatly reduce CPU usage in rooms when there are a lot of people but only a few of\n  them are actively speaking.\n\n\n\n- `integrated_turn_domain`  \n\n  ```\n  binary() | nil\n  ```\n  \n  Default value: `nil`  \n  Domain address of integrated TURN Servers. Required for TLS TURN\n\n\n\n- `integrated_turn_options`  \n\n  ```\n  Membrane.ICE.Endpoint.integrated_turn_options_t()\n  ```\n  \n  Default value: `[]`  \n  \n\n\n\n- `owner`  \n\n  ```\n  pid()\n  ```\n  \n  ***Required***  \n  Pid of parent all notifications will be send to.\n  \n  To see possible notifications please refer to module docs.\n\n\n\n- `video_tracks_limit`  \n\n  ```\n  integer() | nil\n  ```\n  \n  Default value: `nil`  \n  Maximal number of video tracks that will be sent to the the browser at the same time.\n  \n  This variable indicates how many video tracks should be sent to the browser at the same time.\n  If `nil` all video tracks this `Membrane.RTC.Engine.Endpoint.WebRTC` receives will be sent.\n\n\n\n- `rtcp_receiver_report_interval`  \n\n  ```\n  Membrane.Time.t() | nil\n  ```\n  \n  Default value: `nil`  \n  Receiver reports's generation interval, set to nil to avoid reports generation\n\n\n\n- `rtcp_sender_report_interval`  \n\n  ```\n  Membrane.Time.t() | nil\n  ```\n  \n  Default value: `nil`  \n  Sender reports's generation interval, set to nil to avoid reports generation\n\n\n\n- `simulcast_config`  \n\n  ```\n  SimulcastConfig.t()\n  ```\n  \n  Default value: `%Membrane.RTC.Engine.Endpoint.WebRTC.SimulcastConfig{enabled: false, initial_target_variant: &Membrane.RTC.Engine.Endpoint.WebRTC.SimulcastConfig.initial_target_variant/1}`  \n  Simulcast configuration\n\n\n\n- `metadata`  \n\n  ```\n  any()\n  ```\n  \n  Default value: `nil`  \n  Endpoint metadata\n\n\n\n- `telemetry_label`  \n\n  ```\n  Membrane.TelemetryMetrics.label()\n  ```\n  \n  Default value: `[]`  \n  Label passed to Membrane.TelemetryMetrics functions\n\n\n\n- `toilet_capacity`  \n\n  ```\n  pos_integer()\n  ```\n  \n  Default value: `200`  \n  TrackReceiver toilet capacity","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.html#module-bin-options"},{"type":"module","title":"Pads - Membrane.RTC.Engine.Endpoint.WebRTC","doc":"### `:input`\n\nAccepted formats:\n```\n_any\n```\n\nDirection: | `:input`\nAvailability: | `:on_request`\n\n\n### `:output`\n\nAccepted formats:\n```\n_any\n```\n\nDirection: | `:output`\nAvailability: | `:on_request`","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.html#module-pads"},{"type":"function","title":"Membrane.RTC.Engine.Endpoint.WebRTC.options/0","doc":"Returns description of options available for this module","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.html#options/0"},{"type":"function","title":"Membrane.RTC.Engine.Endpoint.WebRTC.to_rid/1","doc":"","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.html#to_rid/1"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.encoding_t/0","doc":"","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.html#t:encoding_t/0"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.media_event_message_t/0","doc":"Type describing message format used to send media events both to the business logic\nand by the business logic to this endpoint.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.html#t:media_event_message_t/0"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.t/0","doc":"Struct containing options for `Membrane.RTC.Engine.Endpoint.WebRTC`","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.html#t:t/0"},{"type":"behaviour","title":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator","doc":"Behavior defining a set of functions for managing connection allocations for TrackReceivers.\n\nIt is responsible for allocating connection bandwidth for track receivers and probing the connection.\n\n# Message protocol\n\nIn addition to implementing the behavior, Each implementation of ConnectionAllocator should be aware of the\nfollowing message protocol.\n\n`Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver` already implements proper handling of this message protocol.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html"},{"type":"behaviour","title":"Granting an allocation - Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator","doc":"`t:Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.AllocationGrantedNotification.t/0` message is sent to TrackReceiver when ConnectionAllocator\nwants to grant an allocation. TrackReceiver is obligated to comply. This message can also be used to\nforce-lower the allocation of a particular TrackReceiver, skipping the negotiation flow.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html#module-granting-an-allocation"},{"type":"behaviour","title":"Negotiating lower allocation - Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator","doc":"Whenever it is deemed necessary, ConnectionAllocator can send `t:decrease_allocation_request/0` to\nselected TrackReceiver, to request that it lowers its allocation by unspecified amount.\nUsually, this will mean that TrackReceiver selects a lower variant (if possible).\nIn response, TrackReceiver sends `t:decrease_allocation_request_response/0` to ConnectionAllocator.\nIf the request was accepted, TrackReceiver is obligated to request lower allocation\nimmediately after sending a reply.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html#module-negotiating-lower-allocation"},{"type":"behaviour","title":"Example of decrease allocation request message flow - Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator","doc":"```\nCA - Connection Allocator\nTR - Track Receiver\n\nCA -> TR: :decrease_your_allocation\nTR -> CA: {:decrease_allocation_request, :accept}\nTR calls `request_allocation/2` with lowered allocation\nCA -> AllocationGrantedNotification\n```","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html#module-example-of-decrease-allocation-request-message-flow"},{"type":"callback","title":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.buffer_sent/2","doc":"Function invoked by the TrackReceiver whenever a buffer is sent","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html#c:buffer_sent/2"},{"type":"callback","title":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.create/0","doc":"A function that should be used by the WebRTC Endpoint to create an instance of ConnectionAllocator","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html#c:create/0"},{"type":"callback","title":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.destroy/1","doc":"A function that should be used by the endpoint to destroy an instance of ConnectionAllocator","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html#c:destroy/1"},{"type":"callback","title":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.register_track_receiver/4","doc":"Function called by the TrackReceiver to register itself in the allocator","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html#c:register_track_receiver/4"},{"type":"callback","title":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.request_allocation/2","doc":"A function used by the VariantSelector to request a different bandwidth allocation.\n\nIn response, the Allocator sends `Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.AllocationGrantedNotification`\nto the track receiver that gets new allocation.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html#c:request_allocation/2"},{"type":"callback","title":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.set_negotiability_status/2","doc":"Function used to change the negotiability status of the TrackReceiver.\n\nTrackReceiver is considered negotiable if it is both capable of decreasing its bandwidth usage and, in principal, allowed to do so.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html#c:set_negotiability_status/2"},{"type":"callback","title":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.update_bandwidth_estimation/2","doc":"A function called by the endpoint, to update the bandwidth estimation in the allocator","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html#c:update_bandwidth_estimation/2"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.allocator_ref/0","doc":"","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html#t:allocator_ref/0"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.decrease_allocation_request/0","doc":"Type describing a message sent by ConnectionAllocator to `Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver`\nto request that they lower their allocation.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html#t:decrease_allocation_request/0"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.decrease_allocation_request_response/0","doc":"Type describing a message sent by  `Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver` to ConnectionAllocator\nin response to `t:decrease_allocation_request/0`, in order to either accept or reject the request.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.html#t:decrease_allocation_request_response/0"},{"type":"module","title":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.AllocationGrantedNotification","doc":"Message sent by ConnectionProber to TrackReceiver whenever new allocation is granted","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.AllocationGrantedNotification.html"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.AllocationGrantedNotification.t/0","doc":"","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator.AllocationGrantedNotification.html#t:t/0"},{"type":"module","title":"Membrane.RTC.Engine.Endpoint.WebRTC.Metrics","doc":"Defines list of metrics, that Reporter instance can aggregate by listening on events emitted in RTC Engine.\nSuggested Reporter implementation is `Membrane.TelemetryMetrics.Reporter` from\n[`membrane_telemetry_metrics`](https://github.com/membraneframework/membrane_telemetry_metrics).\nYou can see usage example in [`membrane_videoroom`](https://github.com/membraneframework/membrane_videoroom).\n\nSee also `Metrics` guide for more details and example report that you can obtain with\n`Membrane.TelemetryMetrics.Reporter`.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.Metrics.html"},{"type":"function","title":"Membrane.RTC.Engine.Endpoint.WebRTC.Metrics.metrics/0","doc":"","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.Metrics.html#metrics/0"},{"type":"module","title":"Membrane.RTC.Engine.Endpoint.WebRTC.NoOpConnectionAllocator","doc":"Implementation of `Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator` that grants all allocations immediately.\n\nIt might be useful for non-WebRTC Endpoints","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.NoOpConnectionAllocator.html"},{"type":"module","title":"Membrane.RTC.Engine.Endpoint.WebRTC.RTPConnectionAllocator","doc":"Implementation of `Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator` that probes the connection\nwith RTP padding packets and grants allocations based on GCC estimates.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.RTPConnectionAllocator.html"},{"type":"function","title":"Membrane.RTC.Engine.Endpoint.WebRTC.RTPConnectionAllocator.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.RTPConnectionAllocator.html#child_spec/1"},{"type":"opaque","title":"Membrane.RTC.Engine.Endpoint.WebRTC.RTPConnectionAllocator.t/0","doc":"","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.RTPConnectionAllocator.html#t:t/0"},{"type":"module","title":"Membrane.RTC.Engine.Endpoint.WebRTC.SimulcastConfig","doc":"Module representing simulcast configuration for WebRTC endpoint.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.SimulcastConfig.html"},{"type":"function","title":"Membrane.RTC.Engine.Endpoint.WebRTC.SimulcastConfig.initial_target_variant/1","doc":"Default implementation of `initial_target_variant` function in `t:t/0`.\n\nReturns :high, which will result in choosing the highest possible encoding.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.SimulcastConfig.html#initial_target_variant/1"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.SimulcastConfig.t/0","doc":"Simulcast configuration.\n\n* `enabled` - whether to accept simulcast tracks or not.\nSetting this to false will result in rejecting all incoming\nsimulcast tracks i.e. client will not send them.\n* `initial_target_variant` - function used to determine initial\ntarget variant this endpoint is willing to receive for given track.\nIt is called for each track this endpoint subscribes for.\nIf not provided, the highest possible variant will be used.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.SimulcastConfig.html#t:t/0"},{"type":"module","title":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver","doc":"Element responsible for handling WebRTC track.\n\nIts main responsibility is to request the highest available\ntrack variant. If currently used track variant becomes inactive,\nTrackReceiver will switch to the next available variant.\n\nOutgoing RTP packets belong to the same sequence number\nand timestamp spaces but they are not guaranteed to be in order and\ncontiguous.\n\nTo unpack RTP see `Membrane.RTC.Engine.Track.get_depayloader/1`.\n\nTo control TrackReceiver behavior see `t:control_messages/0`.\n\nTrackReceiver also emits some notifications. They are defined\nin `t:notifications/0`.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.html"},{"type":"module","title":"Element options - Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver","doc":"Passed via struct `t:Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.t/0`\n\n- `track`  \n\n  ```\n  Track.t()\n  ```\n  \n  ***Required***  \n  Track this adapter will maintain\n\n\n- `initial_target_variant`  \n\n  ```\n  Membrane.RTC.Engine.Track.variant()\n  ```\n  \n  ***Required***  \n  Track variant that will be forwarded whenever it is active.\n  Can be changed with `t:set_target_variant_msg/0`.\n\n\n\n- `keyframe_request_interval`  \n\n  ```\n  Membrane.Time.t() | nil\n  ```\n  \n  Default value: `nil`  \n  Defines how often keyframe requests should be sent for\n  currently used track variant.\n  \n  This option should be used in very specific cases (e.g. see HLS endpoint)\n  as generating keyframes increases track bitrate and might introduce\n  additional delay.\n\n\n\n- `connection_allocator_module`  \n\n  ```\n  module()\n  ```\n  \n  Default value: `Membrane.RTC.Engine.Endpoint.WebRTC.NoOpConnectionAllocator`  \n  Module implementing `Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator` behavior\n  that should be used by the TrackReceiver.\n\n\n\n- `connection_allocator`  \n\n  ```\n  pid() | nil\n  ```\n  \n  Default value: `nil`  \n  PID of the instance of the ConnectionAllocator that should be used by the TrackReceiver\n\n\n\n- `allocation_negotiable?`  \n\n  ```\n  boolean()\n  ```\n  \n  Default value: `false`  \n  Option defining whether allocation for this Track Receiver should be negotiable.\n  \n  Trying to enable negotiability for tracks that are inherently non-negotiable, also\n  known as non-simulcast tracks, will result in a crash.\n  \n  This value can later be changed by sending a `set_negotiable?/0` control message to this Element.\n\n\n\n- `telemetry_label`  \n\n  ```\n  Membrane.TelemetryMetrics.label()\n  ```\n  \n  Default value: `[]`  \n  Label passed to Membrane.TelemetryMetrics functions","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.html#module-element-options"},{"type":"module","title":"Pads - Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver","doc":"### `:input`\n\nAccepted formats:\n```\nMembrane.RTP\n```\n\nDirection: | `:input`\nAvailability: | `:always`\nFlow control: | `:push`\n\n\n### `:output`\n\nAccepted formats:\n```\nMembrane.RTP\n```\n\nDirection: | `:output`\nAvailability: | `:always`\nFlow control: | `:push`","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.html#module-pads"},{"type":"function","title":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.options/0","doc":"Returns description of options available for this module","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.html#options/0"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.control_messages/0","doc":"Messages that can be sent to TrackReceiver to control its behavior.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.html#t:control_messages/0"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.notifications/0","doc":"Notifications that TrackReceiver emits.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.html#t:notifications/0"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.set_negotiable?/0","doc":"Changes negotiability status of the TrackReceiver.\n\nNegotiability refers to the setting in `Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator`\nthat determines if the allocation for the TrackReceiver can be negotiated.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.html#t:set_negotiable?/0"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.set_target_variant/0","doc":"Changes target track variant.\n\nTarget track variant is variant that will be forwarded\nwhenever it is active.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.html#t:set_target_variant/0"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.t/0","doc":"Struct containing options for `Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver`","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.html#t:t/0"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.variant_switch_reason/0","doc":"Reason of track variant switch.\n\n* `:low_bandwidth` - bandwidth was too low to maintain current track quality\n* `:variant_inactive` - variant has been paused due to inactivity or it has been muted\n* `:set_bandwidth_allocation` - variant has been directly set\n* `:variant_resumed` - variant became active *AND* we have enough bandwidth to support it\n* `:variant_unmuted` - previously muted variant has become active\n* `:automatic_selection` - variant became active but no target variant has been set so automatic selection has been performed\n* `:target_variant_selected` - variant has been selected as target and is active at the moment of selection","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.html#t:variant_switch_reason/0"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.variant_switched/0","doc":"Notification emitted whenever TrackReceiver starts receiving a new track variant.","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.html#t:variant_switched/0"},{"type":"type","title":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.voice_activity_changed/0","doc":"Notfication emitted when TrackReceiver receives an update on voice activity","ref":"Membrane.RTC.Engine.Endpoint.WebRTC.TrackReceiver.html#t:voice_activity_changed/0"},{"type":"behaviour","title":"Membrane.WebRTC.Extension","doc":"A module that provides mappings between `ExSDP.Attribute.Extmap` and modules implementing\n`Membrane.WebRTC.Extension` behaviour.","ref":"Membrane.WebRTC.Extension.html"},{"type":"callback","title":"Membrane.WebRTC.Extension.add_to_media/4","doc":"Adds information about extension to an SDP media.","ref":"Membrane.WebRTC.Extension.html#c:add_to_media/4"},{"type":"function","title":"Membrane.WebRTC.Extension.add_to_media/5","doc":"Given an SDP media, a list of supported extensions and supported `Extmap`s, adds corresponding\nextensions to the media.","ref":"Membrane.WebRTC.Extension.html#add_to_media/5"},{"type":"function","title":"Membrane.WebRTC.Extension.as_rtp_extension/3","doc":"Given a list of supported extensions, maps a supported `Extmap` to an `RTP.SessionBin.rtp_extension_t()`.\n\n`:inbound`/`:outbound` denotes track type `extmap` was specified for.","ref":"Membrane.WebRTC.Extension.html#as_rtp_extension/3"},{"type":"function","title":"Membrane.WebRTC.Extension.as_rtp_mapping/2","doc":"Given a list of supported extensions and a supported `Extmap`, generates a mapping from a name provided\nby an extension to an ID provided by the `Extmap`.","ref":"Membrane.WebRTC.Extension.html#as_rtp_mapping/2"},{"type":"callback","title":"Membrane.WebRTC.Extension.compatible?/1","doc":"Returns a boolean indicating whether an extension is compatible with given encoding.","ref":"Membrane.WebRTC.Extension.html#c:compatible?/1"},{"type":"function","title":"Membrane.WebRTC.Extension.from_extmap/2","doc":"Given a list of supported extensions, returns an extension that corresponds to given `Extmap`\nor `:not_supported` if there is no such extension.","ref":"Membrane.WebRTC.Extension.html#from_extmap/2"},{"type":"callback","title":"Membrane.WebRTC.Extension.get_rtp_module/3","doc":"Returns a module that implements the extension in `Membrane.RTP.SessionBin` or `:no_rtp_module` if such\nsuch module would only forward buffers to next element.\n\n`:inbound`/`:outbound` denotes track type this extension was specified for.","ref":"Membrane.WebRTC.Extension.html#c:get_rtp_module/3"},{"type":"callback","title":"Membrane.WebRTC.Extension.new/1","doc":"Creates a `Membrane.WebRTC.Extension` struct for a specific extension.","ref":"Membrane.WebRTC.Extension.html#c:new/1"},{"type":"function","title":"Membrane.WebRTC.Extension.supported?/3","doc":"Given a list of supported extensions, checks if there is an extension that corresponds to\ngiven `Extmap` and encoding.","ref":"Membrane.WebRTC.Extension.html#supported?/3"},{"type":"callback","title":"Membrane.WebRTC.Extension.uri/0","doc":"Returns an URI that identifies extension in SDP","ref":"Membrane.WebRTC.Extension.html#c:uri/0"},{"type":"type","title":"Membrane.WebRTC.Extension.maybe_t/0","doc":"","ref":"Membrane.WebRTC.Extension.html#t:maybe_t/0"},{"type":"type","title":"Membrane.WebRTC.Extension.t/0","doc":"* module - extension specific module which implement `Membrane.WebRTC.Extension` behaviour\n  * rtp_opts - options for RTP module. This allows configuring RTP module to your needs\n  * uri - a URI that identifies extension in SDP\n  * name - an atom identifying the extension in `Membrane.RTP.SessionBin`","ref":"Membrane.WebRTC.Extension.html#t:t/0"},{"type":"module","title":"Membrane.WebRTC.Extension.Mid","doc":"Module implementing `Membrane.WebRTC.Extension` behaviour for Media Identification RTP Header extension.\n\nThis extension is described at https://tools.ietf.org/pdf/draft-ietf-mmusic-rfc8843bis-09.pdf.","ref":"Membrane.WebRTC.Extension.Mid.html"},{"type":"module","title":"Membrane.WebRTC.Extension.RepairedRid","doc":"Module implementing `Membrane.WebRTC.Extension` behaviour for Repaired RTP Stream Identifier RTP Header extension.\n\nThis extension is described in RFC 8852.","ref":"Membrane.WebRTC.Extension.RepairedRid.html"},{"type":"module","title":"Membrane.WebRTC.Extension.Rid","doc":"Module implementing `Membrane.WebRTC.Extension` behaviour for RTP Stream Identifier RTP Header extension.\n\nThis extension is described in RFC 8852.","ref":"Membrane.WebRTC.Extension.Rid.html"},{"type":"module","title":"Membrane.WebRTC.Extension.TWCC","doc":"Module implementing `Membrane.WebRTC.Extension` behaviour for Transport-wide Congestion Control RTP Header extension.\n\nThis extension is described at https://datatracker.ietf.org/doc/html/draft-holmer-rmcat-transport-wide-cc-extensions-01.","ref":"Membrane.WebRTC.Extension.TWCC.html"},{"type":"module","title":"Membrane.WebRTC.Extension.VAD","doc":"Module implementing `Membrane.WebRTC.Extension` behaviour for Client-to-Mixer Audio Level Indication RTP Header extension.\n\nThis extension is described in RFC 6464.","ref":"Membrane.WebRTC.Extension.VAD.html"},{"type":"module","title":"Membrane.WebRTC.EndpointBin","doc":"Module responsible for interacting with a WebRTC peer.\n\nNew tracks are specified by SDP negotiation conducted by messages\n`t:signal_message/0`, and then linking corresponding\n`:input` and `:output` pads with ids reported via `t:new_track_notification/0`\n\nThe tracks can be manipulated by sending `t:track_message/0`.\n\nTo initiate or modify the connection, the bin sends and expects to receive\n`t:signal_message/0`.","ref":"Membrane.WebRTC.EndpointBin.html"},{"type":"module","title":"Bin options - Membrane.WebRTC.EndpointBin","doc":"Passed via struct `t:Membrane.WebRTC.EndpointBin.t/0`\n\n- `direction`  \n\n  ```\n  direction()\n  ```\n  \n  Default value: `:sendrecv`  \n  Direction of EndpointBin. Determines whether\n  EndpointBin can send, receive or both send and receive media.\n  For more information refer to t:direction/0.\n\n\n- `handshake_opts`  \n\n  ```\n  Keyword.t()\n  ```\n  \n  Default value: `[]`  \n  Keyword list with options for handshake module. For more information please\n  refer to `t:ExDTLS.opts_t/0`\n\n\n\n- `rtcp_receiver_report_interval`  \n\n  ```\n  Membrane.Time.t() | nil\n  ```\n  \n  Default value: `nil`  \n  Receiver reports's generation interval, set to nil to avoid reports generation\n\n\n\n- `rtcp_sender_report_interval`  \n\n  ```\n  Membrane.Time.t() | nil\n  ```\n  \n  Default value: `nil`  \n  Sender reports's generation interval, set to nil to avoid reports generation\n\n\n\n- `filter_codecs`  \n\n  ```\n  (Membrane.WebRTC.Track.Encoding.t() -> boolean())\n  ```\n  \n  Default value: `&Membrane.WebRTC.SDP.filter_encodings/1`  \n  Defines function which will filter SDP m-line by codecs\n\n\n\n- `extensions`  \n\n  ```\n  [Extension.t()]\n  ```\n  \n  Default value: `[]`  \n  List of WebRTC extensions that should be enabled\n\n\n\n- `log_metadata`  \n\n  ```\n  :list\n  ```\n  \n  Default value: `[]`  \n  Logger metadata used for endpoint bin and all its descendants\n\n\n\n- `integrated_turn_options`  \n\n  ```\n  [ICE.Endpoint.integrated_turn_options_t()]\n  ```\n  \n  Default value: `[]`  \n  Integrated TURN Options\n\n\n\n- `simulcast?`  \n\n  ```\n  boolean()\n  ```\n  \n  Default value: `true`  \n  Whether to accept simulcast tracks or not.\n  If set to `false`, simulcast tracks will be disabled i.e.\n  sender will not send them.\n\n\n\n- `telemetry_label`  \n\n  ```\n  Membrane.TelemetryMetrics.label()\n  ```\n  \n  Default value: `[]`  \n  Label passed to Membrane.TelemetryMetrics functions","ref":"Membrane.WebRTC.EndpointBin.html#module-bin-options"},{"type":"module","title":"Pads - Membrane.WebRTC.EndpointBin","doc":"### `:input`\n\nAccepted formats:\n```\n_any\n```\n\nDirection: | `:input`\nAvailability: | `:on_request`\nPad options:\n\n- `use_payloader?`  \n\n  ```\n  boolean()\n  ```\n  \n  Default value: `true`  \n  Defines if incoming stream should be payloaded based on given encoding.\n  Otherwise the stream is assumed  be in RTP format.\n\n\n\n### `:output`\n\nAccepted formats:\n```\n_any\n```\n\nDirection: | `:output`\nAvailability: | `:on_request`\nPad options:\n\n- `extensions`  \n\n  ```\n  [Membrane.RTP.SessionBin.extension_t()]\n  ```\n  \n  Default value: `[]`  \n  List of general extensions that will be applied to the SessionBin's output pad\n\n\n- `use_depayloader?`  \n\n  ```\n  boolean()\n  ```\n  \n  Default value: `true`  \n  Defines if the outgoing stream should get depayloaded.\n  \n  This option should be used as a convenience, it is not necessary as the new track notification\n  returns a depayloading filter's definition that can be attached to the output pad\n  to work the same way as with the option set to true.","ref":"Membrane.WebRTC.EndpointBin.html#module-pads"},{"type":"function","title":"Membrane.WebRTC.EndpointBin.options/0","doc":"Returns description of options available for this module","ref":"Membrane.WebRTC.EndpointBin.html#options/0"},{"type":"type","title":"Membrane.WebRTC.EndpointBin.alter_tracks_message/0","doc":"Message that adds or removes tracks.","ref":"Membrane.WebRTC.EndpointBin.html#t:alter_tracks_message/0"},{"type":"type","title":"Membrane.WebRTC.EndpointBin.direction/0","doc":"Type describing possible media flow directions.\n\n* `:recvonly` - only receive media from the peer\n* `:sendonly` - only send media to the peer\n* `:sendrecv` - both send and receive media from the peer","ref":"Membrane.WebRTC.EndpointBin.html#t:direction/0"},{"type":"type","title":"Membrane.WebRTC.EndpointBin.input_pad_opts/0","doc":"Options for pad `:input`","ref":"Membrane.WebRTC.EndpointBin.html#t:input_pad_opts/0"},{"type":"type","title":"Membrane.WebRTC.EndpointBin.new_track_notification/0","doc":"","ref":"Membrane.WebRTC.EndpointBin.html#t:new_track_notification/0"},{"type":"type","title":"Membrane.WebRTC.EndpointBin.output_pad_opts/0","doc":"Options for pad `:output`","ref":"Membrane.WebRTC.EndpointBin.html#t:output_pad_opts/0"},{"type":"type","title":"Membrane.WebRTC.EndpointBin.signal_message/0","doc":"","ref":"Membrane.WebRTC.EndpointBin.html#t:signal_message/0"},{"type":"type","title":"Membrane.WebRTC.EndpointBin.t/0","doc":"Struct containing options for `Membrane.WebRTC.EndpointBin`","ref":"Membrane.WebRTC.EndpointBin.html#t:t/0"},{"type":"type","title":"Membrane.WebRTC.EndpointBin.track_message/0","doc":"","ref":"Membrane.WebRTC.EndpointBin.html#t:track_message/0"},{"type":"module","title":"Membrane.WebRTC.Metrics","doc":"Defines a list of metrics that can be aggregated based on events from `membrane_webrtc_plugin`.","ref":"Membrane.WebRTC.Metrics.html"},{"type":"function","title":"Membrane.WebRTC.Metrics.metrics/0","doc":"","ref":"Membrane.WebRTC.Metrics.html#metrics/0"},{"type":"module","title":"Membrane.WebRTC.Track","doc":"Module representing a WebRTC track.","ref":"Membrane.WebRTC.Track.html"},{"type":"function","title":"Membrane.WebRTC.Track.from_sdp_media/1","doc":"Creates `t:t/0` from SDP m-line with random track id and stream id.","ref":"Membrane.WebRTC.Track.html#from_sdp_media/1"},{"type":"function","title":"Membrane.WebRTC.Track.from_sdp_media/3","doc":"Creates `t:t/0` from SDP m-line with specific track id and stream id.","ref":"Membrane.WebRTC.Track.html#from_sdp_media/3"},{"type":"function","title":"Membrane.WebRTC.Track.new/3","doc":"Creates a new track.\n\nTracks belonging to the same stream should have the same `stream_id`,\nthat can be generated with `stream_id/0`.","ref":"Membrane.WebRTC.Track.html#new/3"},{"type":"function","title":"Membrane.WebRTC.Track.set_constraints/3","doc":"Modifies track properties according to provided constraints.\n\nIn particular, after setting constraints, track can be disabled.","ref":"Membrane.WebRTC.Track.html#set_constraints/3"},{"type":"function","title":"Membrane.WebRTC.Track.simulcast?/1","doc":"Determines if the track is using simulcast","ref":"Membrane.WebRTC.Track.html#simulcast?/1"},{"type":"function","title":"Membrane.WebRTC.Track.stream_id/0","doc":"Generates stream id, that can be used to mark tracks belonging to the same stream.","ref":"Membrane.WebRTC.Track.html#stream_id/0"},{"type":"function","title":"Membrane.WebRTC.Track.to_otel_data/1","doc":"","ref":"Membrane.WebRTC.Track.html#to_otel_data/1"},{"type":"type","title":"Membrane.WebRTC.Track.encoding_key/0","doc":"","ref":"Membrane.WebRTC.Track.html#t:encoding_key/0"},{"type":"type","title":"Membrane.WebRTC.Track.id/0","doc":"","ref":"Membrane.WebRTC.Track.html#t:id/0"},{"type":"type","title":"Membrane.WebRTC.Track.rid/0","doc":"","ref":"Membrane.WebRTC.Track.html#t:rid/0"},{"type":"type","title":"Membrane.WebRTC.Track.status/0","doc":"","ref":"Membrane.WebRTC.Track.html#t:status/0"},{"type":"type","title":"Membrane.WebRTC.Track.t/0","doc":"","ref":"Membrane.WebRTC.Track.html#t:t/0"},{"type":"module","title":"Membrane.WebRTC.Track.Encoding","doc":"A struct holding data passed via SDP about a possible encoding of a track","ref":"Membrane.WebRTC.Track.Encoding.html"},{"type":"type","title":"Membrane.WebRTC.Track.Encoding.t/0","doc":"","ref":"Membrane.WebRTC.Track.Encoding.html#t:t/0"},{"type":"extras","title":"Membrane RTC Engine WebRTC Endpoint","doc":"# Membrane RTC Engine WebRTC Endpoint\n\n[![Hex.pm](https://img.shields.io/hexpm/v/membrane_rtc_engine_webrtc.svg)](https://hex.pm/packages/membrane_rtc_engine_webrtc)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_rtc_engine_webrtc)\n[![codecov](https://codecov.io/gh/jellyfish-dev/membrane_rtc_engine/branch/master/graph/badge.svg?token=9F1XHHUY2B)](https://codecov.io/gh/jellyfish-dev/membrane_rtc_engine)\n[![CircleCI](https://circleci.com/gh/fishjam-dev/membrane_rtc_engine.svg?style=svg)](https://circleci.com/gh/fishjam-dev/membrane_rtc_engine)\n\nWebRTC Endpoint for the [Membrane RTC Engine](https://github.com/jellyfish-dev/membrane_rtc_engine)","ref":"readme.html"},{"type":"extras","title":"Client SDKs - Membrane RTC Engine WebRTC Endpoint","doc":"- [JS/TS](https://github.com/jellyfish-dev/membrane-webrtc-js)\n- [Android](https://github.com/jellyfish-dev/membrane-webrtc-android)\n- [iOS](https://github.com/jellyfish-dev/membrane-webrtc-ios)\n- [React Native](https://github.com/jellyfish-dev/react-native-membrane-webrtc)","ref":"readme.html#client-sdks"},{"type":"extras","title":"Installation - Membrane RTC Engine WebRTC Endpoint","doc":"The package can be installed by adding `membrane_rtc_engine_webrtc` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:membrane_rtc_engine_webrtc, \"~> 0.9.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Developing - Membrane RTC Engine WebRTC Endpoint","doc":"Integration tests using the `test_videoroom` app can be run by executing `mix test.webrtc.integration`\nin the `membrane_rtc_engine` package.\n\nAlternatively, you can use the commands specified [here](integration_test/test_videoroom/README.md).","ref":"readme.html#developing"},{"type":"extras","title":"Copyright and License - Membrane RTC Engine WebRTC Endpoint","doc":"Copyright 2023, [Software Mansion](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane_rtc_engine)\n\n[![Software Mansion](https://logo.swmansion.com/logo?color=white&variant=desktop&width=200&tag=membrane-github)](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane_rtc_engine)\n\nLicensed under the [Apache License, Version 2.0](LICENSE)","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.9.0 - Changelog","doc":"* Add RTCP sender reports [#393](https://github.com/fishjam-dev/membrane_rtc_engine/pull/393)\n* Allow for muting a track without renegotiation [#392](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/392)\n* RTP connection allocator is started without link [#396](https://github.com/fishjam-dev/membrane_rtc_engine/pull/396) \n* Fix handling media events regarding track variant enabling/disabling[#400](https://github.com/fishjam-dev/membrane_rtc_engine/pull/400) \n* Add more logs about signaling [#404](https://github.com/fishjam-dev/membrane_rtc_engine/pull/404)\n* Fix track update race condition [#408](https://github.com/fishjam-dev/membrane_rtc_engine/pull/408)\n* Update dependencies [#2](https://github.com/fishjam-cloud/membrane_rtc_engine/pull/2)","ref":"changelog.html#0-9-0"},{"type":"extras","title":"0.8.0 - Changelog","doc":"* Update deps [#374](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/374)\n* Update the use of Engine.subscribe [#381](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/381)\n* Merge membrane_webrtc_plugin [#384](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/384)","ref":"changelog.html#0-8-0"},{"type":"extras","title":"0.7.0 - Changelog","doc":"* Rename the function `is_keyframe` to `keyframe?` and `is_deficient?` to `deficient?` in order to be compliant with elixir style guide. [#349](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/349)\n* Add support for trackEncodingEnabled and trackEncodingDisabled media events [#352](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/352)\n* Fix TrackRemoved not being sent when Engine subscription failed [#358](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/358)","ref":"changelog.html#0-7-0"},{"type":"extras","title":"0.6.0 - Changelog","doc":"* Remove code related to OpenTelemetry [#340](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/340)","ref":"changelog.html#0-6-0"},{"type":"extras","title":"0.5.0 - Changelog","doc":"* Update to Membrane Core 1.0 [#331](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/331)\n* Change some logs to debug [#327](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/327)\n* Update deps [#333](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/333)","ref":"changelog.html#0-5-0"},{"type":"extras","title":"0.4.0 - Changelog","doc":"* Bump deps [#323](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/323)","ref":"changelog.html#0-4-0"},{"type":"extras","title":"0.3.0 - Changelog","doc":"* Bump deps [#318](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/318)\n* Extend `connected` and `tracksAdded` media events with information about simulcast config. [#317](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/317)","ref":"changelog.html#0-3-0"},{"type":"extras","title":"0.2.1 - Changelog","doc":"* Fix `endpoint_id` missing in WebRTC endpoint telemetry label [#315](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/315/)","ref":"changelog.html#0-2-1"},{"type":"extras","title":"0.2.0 - Changelog","doc":"* Move metrics from the RTC Engine [#306](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/306)\n* Remove unused dependency `membrane_realtimer_plugin` [#300](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/300/)\n* Bump deps [#309](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/309) [#311](https://github.com/jellyfish-dev/membrane_rtc_engine/pull/311)","ref":"changelog.html#0-2-0"},{"type":"extras","title":"0.1.0 - Changelog","doc":"* Initial release","ref":"changelog.html#0-1-0"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2021 Software Mansion\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"},{"type":"extras","title":"Simulcast","doc":"# Simulcast\n\nSimulcast is a technique where a client sends multiple encodings of the same video to the server and the server is responsible for choosing and forwarding proper encoding to proper receiver (other client). The encoding selection is dynamic (i.e. SFU switches between encodings in time) and it is based on:\n\n* receiver available bandwidth\n* receiver preferences (e.g. explicit request to receive video in HD resolution instead of FHD)\n* UI layout (e.g. videos being displayed in smaller video tiles will be sent in a lower resolution)\n\nAt the moment, Membrane supports only receiver preferences i.e. receiver can chose which encoding it is willing to receive. Additionally, sender can turn off/on specific encoding. Membrane RTC Engine will detect changes and switch to another available encoding.","ref":"simulcast.html"},{"type":"extras","title":"Turning simulcast on/off - Simulcast","doc":"","ref":"simulcast.html#turning-simulcast-on-off"},{"type":"extras","title":"Server side - Simulcast","doc":"On the server side, simulcast can be configured while adding new WebRTC Endpoint by setting its `simulcast_config` option.\nRemember to ensure `Rid`, `Mid` and `TWCC` extensions are used.\n\nFor example\n\n```elixir\nalias Membrane.RTC.Engine.Endpoint.WebRTC\nalias Membrane.RTC.Engine.Endpoint.WebRTC.SimulcastConfig\nalias Membrane.WebRTC.Extension.{Mid, Rid, TWCC}\n# ...\n%WebRTC{\n  rtc_engine: rtc_engine,\n  # ...\n  webrtc_extensions: [Rid, Mid, TWCC],\n  simulcast_config: %SimulcastConfig{\n    enabled: true,\n    initial_target_variant: fn _track_ -> :medium end\n  }\n}\n```\n\nHere we turn simulcast on and choose `:medium` variant as a target for each track\nthat is going to be forwarded to the browser.\nTarget variant means that it will be chosen whenever it is active.\nIf for some reason `:medium` variant is inactive we will temporarily forward some other variant.\n\nOn the other hand, setting `enabled` to `false` will result in rejecting all incoming simulcast tracks i.e. client will not send them to the server.","ref":"simulcast.html#server-side"},{"type":"extras","title":"Client side - Simulcast","doc":"On the client side simulcast can be enabled while adding a new track and might differ depending\non the client type e.g. for JS see https://docs.membrane.stream/membrane-webrtc-js/classes/membranewebrtc.html#addtrack","ref":"simulcast.html#client-side"},{"type":"extras","title":"Metrics","doc":"# Metrics\n\nWebRTC Endpoint uses [`membrane_telemetry_metrics`](https://github.com/membraneframework/membrane_telemetry_metrics) to aggregate data about media streams and generate reports about them.\nTo enable metrics aggregation, you have to put line \n\n```elixir\nconfig :membrane_telemetry_metrics, enabled: true\n```\n\nin your config file, add `telemetry_label` to your endpoint configuration\n\n```elixir\n%Endpoint.WebRTC{\n  telemetry_label: [room_id: room_id]\n}\n```\n\nand start `Membrane.TelemetryMetrics.Reporter` with RTC Engine metrics by calling\n\n```elixir \n{:ok, reporter} = Membrane.TelemetryMetrics.Reporter.start_link(Membrane.RTC.Engine.Endpoint.WebRTC.Metrics.metrics())\n```\n\nThen, if you want to get a report with metrics values for every running RTC Engine on the node, you have to call\n```elixir\nMembrane.TelemetryMetrics.Reporter.scrape(reporter)\n```\n\nThere is a report example below, with only one room with one endpoint inside\n```elixir\n%{\n  {:room_id, \"test\"} => %{\n    {:endpoint_id, \"7eda6931-0313-497e-93a0-6a9540407f77\"} => %{\n      :\"ice.binding_requests_received\" => 3,\n      :\"ice.binding_responses_sent\" => 3,\n      :\"ice.bytes_received\" => 20672,\n      :\"ice.bytes_sent\" => 1634,\n      :\"ice.packets_received\" => 46,\n      :\"ice.packets_sent\" => 3,\n      :\"ice.port\" => 51895,\n      :\"ice.protocol\" => :udp,\n      :\"peer.metadata\" => nil,\n      :\"sdp.answer\" => \"v=0\\r\\no=- ...\", \n      :\"sdp.offer\" => \"v=0\\r\\no=- ...\", \n      {:track_id,\n       \"7eda6931-0313-497e-93a0-6a9540407f77:3d228c10-d3b9-4009-b14f-4b0f2b89f7ba:l\"} => %{\n        \"inbound-rtp.bytes_received\": 6470,\n        \"inbound-rtp.encoding\": :H264,\n        \"inbound-rtp.markers_received\": 4,\n        \"inbound-rtp.packets\": 10,\n        \"inbound-rtp.ssrc\": 1546136873,\n        rtx_stream: %{\n          \"inbound-rtp.bytes_received\": 5120,\n          \"inbound-rtp.encoding\": :rtx,\n          \"inbound-rtp.markers_received\": 1,\n          \"inbound-rtp.packets\": 6,\n          \"inbound-rtp.ssrc\": 84792660\n        },\n        \"track.metadata\": %{\"active\" => true, \"type\" => \"camera\"}\n      },\n      {:track_id,\n       \"7eda6931-0313-497e-93a0-6a9540407f77:3d228c10-d3b9-4009-b14f-4b0f2b89f7ba:m\"} => %{\n        \"inbound-rtp.bytes_received\": 5988,\n        \"inbound-rtp.encoding\": :H264,\n        \"inbound-rtp.packets\": 6,\n        \"inbound-rtp.ssrc\": 3428415963,\n        \"track.metadata\": %{\"active\" => true, \"type\" => \"camera\"}\n      },\n      {:track_id,\n       \"7eda6931-0313-497e-93a0-6a9540407f77:90ce43b1-d37a-452e-8a04-b2883e7d54dc:\"} => %{\n        \"inbound-rtp.bytes_received\": 1885,\n        \"inbound-rtp.encoding\": :OPUS,\n        \"inbound-rtp.markers_received\": 1,\n        \"inbound-rtp.packets\": 18,\n        \"inbound-rtp.ssrc\": 3178961132,\n        \"rtcp.total_packets_sent\": 1,\n        \"track.metadata\": %{\"active\" => true, \"type\" => \"audio\"}\n      }\n    }\n  }\n}\n```\n\nYou can also go to [`membrane_videoroom`](https://github.com/membraneframework/membrane_videoroom) or [`membrane_telemetry_metrics`](https://github.com/membraneframework/membrane_telemetry_metrics) docs for more examples.","ref":"metrics.html"},{"type":"extras","title":"WebRTC Media Events","doc":"# WebRTC Media Events\n\nThis document described all messages exchanged between the WebRTC Endpoint and a client.\nIn general, each media event has to be sent in the form:\n\n```json\n{\n  type: name,\n  data: data\n}\n```\n\nwhere the name of the event is the value of the `type` field, while `data` should carry the data shown below for every media event","ref":"webrtc_media_events.html"},{"type":"extras","title":"Table of contents - WebRTC Media Events","doc":"Below you can find a summary of all media events defined inside `membrane_rtc_engine`","ref":"webrtc_media_events.html#table-of-contents"},{"type":"extras","title":"Generic WebRTC Endpoint messages - WebRTC Media Events","doc":"Messages used by any WebRTC Endpoint plugin\n\n#### WebRTC Endpoint receives these types of `media_events` from client:\n| Name                                              | Description                                                     |\n| ------------------------------------------------- | --------------------------------------------------------------- |\n| [connect](#connect)                               | sent when client wants to connect to associated WebRTC Endpoint |\n| [disconnect](#disconnect)                         | sent when client disconnects from associated endpoint           |\n| [updateEndpointMetadata](#updateendpointmetadata) | contains new metadata of client's endpoint                      |\n| [updateTrackMetadata](#updatetrackmetadata)       | contains new metadata of a track                                |\n| [disableTrackEncoding](#disabletrackencoding)     | sent when client disables one of the track variants             |\n| [enableTrackEncoding](#enabletrackencoding)       | sent when client enables one of the track variants              |\n| [muteTrack](#mutetrack)                           | sent when track is muted                                        |\n| [unmuteTrack](#unmutetrack)                       | sent when track is unmuted                                      |\n| [custom](#custom)                                 | message forwarded to the endpoint associated with the client    |\n\n\n\n#### WebRTC Endpoint sends these types of messages to the client:\n\n| Name                                            | Description                                                                  |\n| ----------------------------------------------- | ---------------------------------------------------------------------------- |\n| [tracksPriority](#trackspriority)               | contains all tracks that will be forwarded to client until next this message |\n| [endpointUpdated](#endpointupdated)             | contains new metadata of an endpoint                                         |\n| [trackUpdated](#trackupdated)                   | contains new metadata of a track                                             |\n| [tracksAdded](#tracksadded)                     | sent when some tracks were added by an endpoint                              |\n| [tracksRemoved](#tracksremoved)                 | sent when some tracks were removed by an endpoint                            |\n| [trackEncodingDisabled](#trackencodingdisabled) | sent when some track variant were disabled by an endpoint                    |\n| [trackEncodingEnabled](#trackencodingenabled)   | sent when some track variant were enabled by an endpoint                     |\n| [endpointAdded](#endpointadded)                 | sent after after new endpoint was added                                      |\n| [connected](#connected)                         | sent as a response to [connect](#connect) when succesful                     |\n| [endpointRemoved](#endpointremoved)             | sent after some endpoint was removed                                         |\n| [error](#error)                                 | Notifies about an error that occurred                                        |\n| [custom](#custom-1)                             | custom message forwarded from endpoint to client                             |","ref":"webrtc_media_events.html#generic-webrtc-endpoint-messages"},{"type":"extras","title":"WebRTC custom media events - WebRTC Media Events","doc":"#### WebRTC endpoint receives these types of custom `media_event`s from client:\n\n| Name                                            | Description                                                               |\n| ----------------------------------------------- | ------------------------------------------------------------------------- |\n| [renegotiateTracks](#renegotiatetracks)         | A request from a client to renegotiate tracks                             |\n| [prioritizeTrack](#prioritizetrack)             | A request to prioritize the track                                         |\n| [unprioritizeTrack](#unprioritizetrack)         | A request to unprioritize the track                                       |\n| [preferedVideoSizes](#preferedvideosizes)       | informs of how many videos in different quality a client wants to receive |\n| [candidate](#candidate)                         | Contains client's ICE candidate                                           |\n| [sdpOffer](#sdpoffer)                           | Contains an SDP offer from a client                                       |\n| [setTargetTrackVariant](#setTargetTrackVariant) | A request from a client to receive a specific track variant               |\n| [trackVariantBitrates](#trackVariantBitrate)    | Contains updated bitrates for track's variants                            |\n\n#### WebRTC endpoint sends these type of custom messages to client\n| Name                                  | Description                                                       |\n| ------------------------------------- | ----------------------------------------------------------------- |\n| [offerData](#offerdata)               | Sends data needed by the client to create an offer                |\n| [candidate](#candidate-1)             | Contains an ICE candidate                                         |\n| [sdpAnswer](#sdpanswer)               | Provides an SDP Answer to the client's offer                      |\n| [encodingSwitched](#encodingswitched) | An information that a track will be sent with a specific encoding |\n| [vadNotification](#vadnotification)   | An update on Voice Activity Detection                             |","ref":"webrtc_media_events.html#webrtc-custom-media-events"},{"type":"extras","title":"Client -> WebRTC Endpoint - WebRTC Media Events","doc":"### `connect`\n\n* Sent when a client wants to join WebRTC Endpoint. It contains only endpoint's metadata\n\n  ```json\n  {\n    \"metadata\": any\n  }\n  ```\n\n### `disconnect`\n\n* Sent when client disconnects from WebRTC Endpoint. It contains no data\n\n  ```json\n  {}\n  ```\n\n### `updateEndpointMetadata`\n\n* Contains new metadata of client's endpoint\n\n  ```json\n  {\n    \"metadata\": any\n  }\n  ```\n\n### `disableTrackEncoding`\n\n* Sent when client disables one of the track variants\n\n  ```json\n  {\n    \"trackId\": track_id,\n    \"encoding\": encoding\n  }\n  ```\n\n### `enableTrackEncoding`\n\n* Sent when client enables one of the track variants\n\n  ```json\n  {\n    \"trackId\": track_id,\n    \"encoding\": encoding\n  }\n  ```\n\n### `updateTrackMetadata`\n\n* Contains new metadata of some track and an id of this track\n\n  ```json\n  {\n    \"trackId\": track_id,\n    \"trackMetadata\": any\n  }\n  ```\n\n### `muteTrack`\n\n* Sent when a track is muted\n\n  ```json\n  {\n    \"trackId\": track_id,\n  }\n  ```\n\n\n### `unmuteTrack`\n\n* Sent when a track is unmuted\n\n  ```json\n  {\n    \"trackId\": track_id,\n  }\n  ```\n\n\n### `custom`\n\n* A black-box for a message that WebRTC Endpoint will forward to the endpoint associated with the client who sent the message.\n\n  ```json\n  any\n  ```","ref":"webrtc_media_events.html#client-webrtc-endpoint"},{"type":"extras","title":"WebRTC Endpoint -> Client - WebRTC Media Events","doc":"### `tracksPriority`\n\n* Contains all tracks that will be forwarded to client until the next `tracks_priority` message.\n  This type of messages is sent only if DisplayManager is spawned\n\n  ```json\n  {\n    tracks: tracks\n  }\n  ```\n\n### `endpointUpdated`\n\n* Contains information about updated metadata of one of the endpoints\n\n  ```json\n  {\n    endpointId: endpoint_id,\n    metadata: any\n  }\n  ```\n\n### `trackUpdated`\n\n* Contains information about new metadata of one of tracks\n\n  ```json\n  {\n    endpointId: endpoint_id,\n    trackId: track_id,\n    metadata: any\n  }\n  ```\n\n### `tracksAdded`\n\n* Informs that one of the clients has added one or more tracks.\n  It contains:\n  - an id of endpoint associated with that client, \n  - a map of all tracks with `track_id`'s as keys and objects with `track_metadata` and simulcast conifg as a value \n  - (Depracated field use tracks) a map of all tracks with `track_id`s as keys and `track_metadata` as value.\n\n  ```json\n  {\n    endpointId: endpoint_id,\n    trackIdToMetadata: {\n      track_id: any\n    },\n    tracks: {\n      track_id: {\n        metadata: any\n        simulcastConfig: {\n          enabled: boolean,\n          activeEncodings: string[],\n          disabledEncodings: string[]\n        }\n      }\n    }\n  }\n  ```\n\n### `tracksRemoved`\n\n* Contains a list of tracks which have been removed by some client and id of that client's endpoint\n\n  ```json\n  {\n    endpointId: endpoint_id,\n    trackIds: track_ids\n  }\n  ```\n\n### `trackEncodingDisabled`\n\n* Sent when some track variant were disabled by an endpoint\n\n  ```json\n  {\n    \"trackId\": track_id,\n    \"encoding\": encoding\n  }\n  ```\n\n### `trackEncodingEnabled`\n\n* Sent when some track variant were enabled by an endpoint\n\n  ```json\n  {\n    \"trackId\": track_id,\n    \"encoding\": encoding\n  }\n  ```\n\n### `endpointAdded`\n\n* Message sent to all clients in the room after a new endpoint was added.\n  It contains id and metadata of the new endpoint.\n\n  ```json\n  {\n    id: endpoint_id,\n    metadata: any\n  }\n  ```\n\n### `connected`\n\n* Message sent to the client after connecting to the WebRTC Endpoint. It contains the id of that client's endpoint and a list of information about endpoints in the Engine\n(id, metadata, a `trackIdToMetadata` and tracks like seen in `tracksAdded`)\n\n  ```json\n  {\n    id: endpoint_id,\n    otherEndpoints: endpoints\n  }\n  ```\n\n### `endpointRemoved`\n\n* Sent to all remaining clients in the room after some endpoint was removed. It contains an id of the removed endpoint.\n\n  ```json\n  {\n    id: endpoint_id\n  }\n  ```\n\n### `error`\n\n* Informs that an error occurred on the server providing a message to show\n\n  ```json\n  {\n    message: any()\n  }\n  ```\n\n### `custom`\n\n* WARNING!! This type of media event is deprecated - will soon be removed.\n* A black-box for a message that the WebRTC Endpoint will forward to the client.\n\n  ```json\n  any\n  ```","ref":"webrtc_media_events.html#webrtc-endpoint-client"},{"type":"extras","title":"Client -> WebRTC Enpoint - WebRTC Media Events","doc":"### `renegotiateTracks`\n\n* Informs that a client wants to renegotiate connection due to adding a track or removing a track\n\n  ```json\n  {}\n  ```\n\n### `prioritizeTrack`\n\n* Contains an id of the track that the client wants to prioritize\n\n  ```json\n  {\n    trackId: trackId\n  }\n  ```\n\n### `unprioritizeTrack`\n\n* Contains an id of the track that the client want to unprioritize.\n\n  ```json\n  {\n    trackId: trackId\n  }\n  ```\n\n### `preferedVideoSizes`\n\n* Contains information of how many videos in different quality (high, medium, small) the client would like to receive.\n  `allSameSize` flag which indicates that all videos should be in the same quality.\n\n  ```json\n  {\n    bigScreens: Int,\n    mediumScreens: Int,\n    smallScreens: Int,\n    allSameSize: boolean\n  }\n  ```\n\n### `candidate`\n\n* Contains an ICE candidate that is forwarded to the WebRTC Endpoint\n\n  ```json\n  {\n    candidate: candidate,\n    sdpMLineIndex: Int\n  }\n  ```\n\n### `sdpOffer`\n\n* Contains an SDP offer, a mapping between `track_id` and `track_metadata`, mapping between `track_id`\n  and this track's bitrates (or all of its variants bitrates) and a mapping between `mid` and `track_id`.\n  Maps contain only information about current client's `sendonly` tracks.\n\n  ```json\n  {\n    sdpOffer: {\n      type: \"offer\",\n      sdp: sdp_offer\n    },\n    trackIdToTrackMetadata: {\n      trackId: any\n    },\n    trackIdToTrackBitrates: {\n      trackId: number | {rid: number}\n    },\n    midToTrackId: mid_to_track_id\n  }\n  ```\n\n### `setTargetTrackVariant`\n\n* Informs that a client wants to receive a specific track variant.\n  The track variant will be sent whenever it is available.\n  If choosen variant is unavailable, some other variant will be\n  sent until choosen variant becomes active again.\n\n  ```json\n  {\n    trackId => track_id,\n    variant => variant\n  }\n  ```\n\n### `trackVariantBitrates`\n\n* Contains updated bitrates of variants of the track send by the client.\n  Needs to contain all of the variants.\n\n```json\n{\n  trackId: track_id,\n  variantBitrates: {rid: number}\n}\n```","ref":"webrtc_media_events.html#client-webrtc-enpoint"},{"type":"extras","title":"WebRTC Enpoint -> Client - WebRTC Media Events","doc":"### `offerData`\n\n* Contains information about the number  of audio and video tracks that will be sent from the engine\n  to the client and information regarding the integrated TURN server.\n\n  ```json\n  {\n    tracksTypes: {\n      audio: Int,\n      video: Int\n    },\n    integratedTurnServers: [{\n      serverAddr: addr,\n      serverPort: Int,\n      transport: string,\n      password: string,\n      username: string\n    }]\n  }\n  ```\n\n### `candidate`\n\n* Contains information about an ICE candidate which will be sent to the client\n\n  ```json\n  {\n    candidate: candidate,\n    sdpMLineIndex: sdp_m_line_index,\n    sdpMid: nil,\n    usernameFragment: nil\n  }\n  ```\n\n### `sdpAnswer`\n\n* Contains an SDP answer and mapping between `mid` and `track_id` for all tracks (active, inactive, inbound and outbound)\n\n  ```json\n  {\n    type: \"answer\",\n    sdp: sdp_answer,\n    midToTrackId: mid_to_track_id\n  }\n  ```\n\n### `encodingSwitched`\n\n* Informs that track with id `trackId` belonging to endpoint with id `endpointId` will be sent in encoding `encoding` now.\nThe meaning is as follows:\n* \"low_bandwidth\" - we no longer have enough bandwidth to maintain current track quality\n* \"encoding_inactive\" - encoding became inactive\n* \"other\" - it was hard to determine exact reason of encoding switch\n\n  ```json\n  {\n    endpointId: endpoint_id,\n    trackId: track_id,\n    encoding: encoding,\n    reason: \"low_bandwidth\" | \"encoding_inactive\" | \"other\"\n  }\n  ```\n\n### `vadNotification`\n* Informs that the track denoted by `trackId` has changed their voice actiivty\n* For this notification to work, the server must be configured to use VAD extension\n  and the sender must support it.\n\n```json\n{\n  trackId: track_id,\n  status: \"silence\" | \"speech\"\n}\n```\n\n### `bandwidthEstimation`\n* Informs about client's available incoming bitrate (a.k.a. download) estimated by the server.\n  It is measured in bits per second.\n* For this notification to work, the server must be configured to use TWCC extension\n  and the sender must support it.\n\n```json\n{\n  estimation: estimation,\n}\n```","ref":"webrtc_media_events.html#webrtc-enpoint-client"},{"type":"extras","title":"Signaling Protocol","doc":"# Signaling Protocol","ref":"protocol.html"},{"type":"extras","title":"Joining the server - Signaling Protocol","doc":"```mermaid\nsequenceDiagram\n    actor client\n    participant server\n    actor other_client\n    client->>server: connect\n    server->>client: connected\n    server->>other_client: endpointAdded\n```","ref":"protocol.html#joining-the-server"},{"type":"extras","title":"Adding a track - Signaling Protocol","doc":"```mermaid\nsequenceDiagram\n    actor client\n    participant server\n    actor other_client\n    client->>server: renegotiateTracks\n    server->>client: custom(offerData)\n    client->>server: custom(sdpOffer)\n    par\n        client->>server: custom(candidate)\n    and\n        server->>client: custom(sdpAnswer)\n        server->>client: custom(candidate)\n        server->>other_client: tracksAdded\n        server->>client: tracksAdded\n        rect rgb(135, 204, 232)\n            note right of server: renegotiation\n            server->>other_client: custom(offerData)\n            other_client->>server: custom(sdpOffer)\n            par\n                other_client->>server: custom(candidate)\n            and\n                server->>other_client: custom(sdpAnswer)\n                server->>other_client: custom(candidate)\n            end\n        end\n    end\n```","ref":"protocol.html#adding-a-track"},{"type":"extras","title":"Leaving the server - Signaling Protocol","doc":"```mermaid\nsequenceDiagram\n    actor client\n    participant server\n    actor other_client\n    client->>server: disconnect\n    rect rgb(135, 204, 232)\n        note right of server: renegotiation\n        server->>other_client: custom(offerData)\n        other_client->>server: custom(sdpOffer)\n        par\n            other_client->>server: custom(candidate)\n        and\n            server->>other_client: custom(sdpAnswer)\n            server->>other_client: custom(candidate)\n        end\n    end\n```","ref":"protocol.html#leaving-the-server"},{"type":"extras","title":"WebRTC Endpoint","doc":"# WebRTC Endpoint\n\nThe WebRTC Endpoint is responsible for sending/receiving media tracks to/from a web borwser.","ref":"webrtc_endpoint.html"},{"type":"extras","title":"Architecture - WebRTC Endpoint","doc":"![Alt text](assets/webrtc_endpoint_architecture.svg)","ref":"webrtc_endpoint.html#architecture"},{"type":"extras","title":"TrackSender - WebRTC Endpoint","doc":"`TrackSender` is responsible for observing track activity and publishing track data to the engine.\nIn particular, it is responsible for sending `Membrane.RTC.Engine.Event.TrackVariantResumed` \nand `Membrane.RTC.Engine.Event.TrackVariantPaused` events, and replying to `Membrane.KeyframeRequestEvent`.\n\n#### VariantTracker\n\nA browser can pause sending some encoding when e.g. it doesn't have enough bandwidth.\nThis fact is not communicated to the server.\n`VariantTracker` is responsible for tracking variant activity i.e. whether it is still active.","ref":"webrtc_endpoint.html#tracksender"},{"type":"extras","title":"TrackReceiver - WebRTC Endpoint","doc":"`TrackReceiver` is responsible for receiving a track from the engine.\nIn particular, it is responsible for sending the `Membrane.RTC.Engine.Event.RequestTrackVariant` event, \nforwarding keyframe requests from the browser to the `TrackSender` and repairing RTP packets after\nswitching track variants.","ref":"webrtc_endpoint.html#trackreceiver"},{"type":"extras","title":"ConnectionAllocator - WebRTC Endpoint","doc":"`ConnectionAllocator` is responsible for probing the connection and negotiating bandwidth with `TrackReceiver`.\nThe RTC Engine ships with two implementations of `ConnectionAllocator`:\n* `Membrane.RTC.Engine.Endpoint.WebRTC.NoOpConnectionAllocator` - always grants bandwidth requested by `TrackReceiver`\n* `Membrane.RTC.Engine.Endpoint.WebRTC.RTPConnectionAllocator` - probes the connection with RTP padding packets and grants bandwidth basing on GCC estimates\n\nYou can read more in `Membrane.RTC.Engine.Endpoint.WebRTC.ConnectionAllocator` module docs.","ref":"webrtc_endpoint.html#connectionallocator"},{"type":"extras","title":"Simulcast","doc":"# Simulcast\n\nSimulcast is a technique where a client sends multiple encodings of the same video to the server\nand the server is responsbile for choosing and forwarding proper encoding to proper receiver (other client).\nThe encoding selection is dynamic (i.e. SFU switches between encodings in time) and it is based on:\n* receiver awailable bandwidth\n* receiver preferences (e.g. explicit request to receive video in HD resolution instead of FHD)\n* UI layaout (e.g. videos being displayed in smaller video tiles will be sent in lower resolution)\n\nAt the moment, Membrane supports only receiver preferences i.e. receiver can chose which encoding\nit is willing to receive.\nAdditionaly, sender can turn off/on specific encoding. \nMembrane RTC Engine will detect changes and switch to another available encoding.","ref":"internal_simulcast.html"},{"type":"extras","title":"Codecs - Simulcast","doc":"At the moment, both H264 and VP8 codecs are supported.","ref":"internal_simulcast.html#codecs"},{"type":"extras","title":"Logic - Simulcast","doc":"Switching between encodings has to be performed transparently from the receiver point of view.\nThis implies that:\n* server has to rewrite some fields of RTP packets to avoid gaps in sequence numbers or timestamps after encoding switch i.e. receiver has to think it is still receiving the same RTP stream\n* encoding switch has to be performed when server receives a keyframe from the new encoding. \nIn other case video will freeze for a while after switching encodings.","ref":"internal_simulcast.html#logic"},{"type":"extras","title":"Forwarder - Simulcast","doc":"It is responsible for forwarding proper encoding.\nIt encapsulates mungers (described below).","ref":"internal_simulcast.html#forwarder"},{"type":"extras","title":"Mungers - Simulcast","doc":"Mungers are responsible for modyfing RTP packets so that switching between encodings is transparent\nfor the receiver.\nCurrently, there are two mungers: RTP Munger and VP8 Munger. \nH264 doesn't require modyfing its RTP packets so there is no H264 Munger.","ref":"internal_simulcast.html#mungers"},{"type":"extras","title":"RTP Munger - Simulcast","doc":"Rewrites RTP Header fields - sequence numbers and timestamps.\nSSRC is being rewritten by other Membrane element.","ref":"internal_simulcast.html#rtp-munger"},{"type":"extras","title":"VP8 Munger - Simulcast","doc":"Rewrites VP8 RTP payload fields - keyidx, picture_id, tl0picidx","ref":"internal_simulcast.html#vp8-munger"},{"type":"extras","title":"Disabling Simulcast - Simulcast","doc":"Simulcast can be disabled per WebRTC Endpoint.\nAt the moment, if client offers simulcast but we don't accept it, we will\ndisable the whole track.\nThis is not compliant with WebRTC standard as we should only remove SDP attributes \nrelated to simulcast and be prepared for receiving one encoding.\nHowever, in such a case browser changes SSRC after ICE restart and \nwe cannot handle this at the moment.","ref":"internal_simulcast.html#disabling-simulcast"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}